#cut sequences output by nanopore

#general stuff
rm(list=ls()) #ctrl L for clearing the workspace

#PACKAGES TO INSTALL
#install.packages("seqinr") #for the read and write fasta functions

#PACKAGES TO LOAD
library(seqinr)

#VARIABLES TO SET
setwd("/home/gemma/Documents/iGEM/RstudioiGEM/sequencegenerator") #set working directory to where you have your sequence saved and would like to save the generated file
Gene <- "EPOcDNA.fasta" #insert file name HERE. If human genomic data, check that distribution is exponential (rexp)
seqamount <- 20 #amount of sequence cuts to be put in outfile
minlength <- 20 #minimal length of sequences, adapt to the relevant threshold of your sequencer
outfile <- "EPOcDNAcutexp20times.fasta" #name of output file


#####################################################################
############### RUN FROM HERE ON SHOULD BE AUTOMATIC#################

#read fasta sequence and store as vector with individual elements:
Gene <- read.fasta(Gene, as.string = TRUE)
Gene <- as.character(Gene)
Gene <- strsplit(Gene, NULL)
Gene <-unlist(Gene)

#prepare exponential distribution
x1 <- rexp(1000) #make a bucket of numbers with a certain distribution (exponential for human genome)
x5 <- x1*(length(Gene)/max(x1)) #normalise and scale to length Gene
x6 <- round(x5, digits=0) #make a round number
x7 <- x6 >=minlength #set minimum length
expdis <- x6[x7] #vector with below minimum lengths taken out. (~25% for 50bps)

possstarts <-round(runif(10000,0,length(Gene)), digits=0) #generate random values from the uniform distribution that are possible starting points

#cut input sequence
cuts <- vector()
for (i in 1:seqamount){
  lengthcut <- sample(expdis, 1, replace=TRUE) #select a value from the exponential ditribution to find length of cut
  startpos <- sample(possstarts, 1, replace=TRUE)  #select starting position
  cutnum <- startpos:(startpos+lengthcut) #find the whole cut (in numbers)
  cutbps <- Gene[cutnum] #find the whole cut (in bps)
  cuts [i] <-list(cutbps, i) #save the cuts. save per i vector x3 in an entry of cuts
}

write.fasta(sequences = cuts, names = names(cuts), file.out = outfile) #save group of output strands